@@ -2,7 +2,7 @@
 # Code written by Fatih C Akyon, 2020.
 
 import copy
-from typing import Dict, List
+from typing import List, Optional, Dict
 
 import numpy as np
 
@@ -20,12 +20,12 @@ class BoundingBox:
     Bounding box of the annotation.
     """
 
-    def __init__(self, box: list, shift_amount: list = [0, 0]):
+    def __init__(self, box: List[int], shift_amount: List[int] = [0, 0]):
         """
         Args:
-            box: List
+            box: List[int]
                 [minx, miny, maxx, maxy]
-            shift_amount: List
+            shift_amount: List[int]
                 To shift the box and mask predictions from sliced image
                 to full sized image, should be in the form of [shift_x, shift_y]
         """
@@ -162,9 +162,7 @@ class Mask:
         # confirm full_shape is given
         assert full_shape is not None, "full_shape must be provided"
 
-        bool_mask = get_bool_mask_from_coco_segmentation(
-            segmentation, height=full_shape[0], width=full_shape[1]
-        )
+        bool_mask = get_bool_mask_from_coco_segmentation(segmentation, height=full_shape[0], width=full_shape[1])
         return cls(
             bool_mask=bool_mask,
             shift_amount=shift_amount,
@@ -234,9 +232,7 @@ class Mask:
 
     def get_shifted_mask(self):
         # Confirm full_shape is specified
-        assert (self.full_shape_height is not None) and (
-            self.full_shape_width is not None
-        ), "full_shape is None"
+        assert (self.full_shape_height is not None) and (self.full_shape_width is not None), "full_shape is None"
         # init full mask
         mask_fullsized = np.full(
             (
@@ -255,9 +251,7 @@ class Mask:
         ]
 
         # convert sliced mask to full mask
-        mask_fullsized[
-            starting_pixel[1] : ending_pixel[1], starting_pixel[0] : ending_pixel[0]
-        ] = self.bool_mask[
+        mask_fullsized[starting_pixel[1] : ending_pixel[1], starting_pixel[0] : ending_pixel[0]] = self.bool_mask[
             : ending_pixel[1] - starting_pixel[1], : ending_pixel[0] - starting_pixel[0]
         ]
 
@@ -289,10 +283,10 @@ class ObjectAnnotation:
     def from_bool_mask(
         cls,
         bool_mask,
-        category_id=None,
-        category_name=None,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        category_id: Optional[int] = None,
+        category_name: Optional[str] = None,
+        shift_amount: Optional[List[int]] = [0, 0],
+        full_shape: Optional[List[int]] = None,
     ):
         """
         Creates ObjectAnnotation from bool_mask (2D np.ndarray)
@@ -314,7 +308,7 @@ class ObjectAnnotation:
             category_id=category_id,
             bool_mask=bool_mask,
             category_name=category_name,
-            shift_amount=[0, 0],
+            shift_amount=shift_amount,
             full_shape=full_shape,
         )
 
@@ -322,10 +316,10 @@ class ObjectAnnotation:
     def from_coco_segmentation(
         cls,
         segmentation,
-        category_id=None,
-        category_name=None,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        full_shape: List[int],
+        category_id: Optional[int] = None,
+        category_name: Optional[str] = None,
+        shift_amount: Optional[List[int]] = [0, 0],
     ):
         """
         Creates ObjectAnnotation from coco segmentation:
@@ -352,23 +346,23 @@ class ObjectAnnotation:
                 To shift the box and mask predictions from sliced image to full
                 sized image, should be in the form of [shift_x, shift_y]
         """
-        bool_mask = get_bool_mask_from_coco_segmentation(segmentation)
+        bool_mask = get_bool_mask_from_coco_segmentation(segmentation, width=full_shape[1], height=full_shape[0])
         return cls(
             category_id=category_id,
             bool_mask=bool_mask,
             category_name=category_name,
-            shift_amount=[0, 0],
+            shift_amount=shift_amount,
             full_shape=full_shape,
         )
 
     @classmethod
     def from_coco_bbox(
         cls,
-        bbox,
-        category_id=None,
-        category_name=None,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        bbox: List[int],
+        category_id: Optional[int] = None,
+        category_name: Optional[str] = None,
+        shift_amount: Optional[List[int]] = [0, 0],
+        full_shape: Optional[List[int]] = None,
     ):
         """
         Creates ObjectAnnotation from coco bbox [minx, miny, width, height]
@@ -395,18 +389,58 @@ class ObjectAnnotation:
             category_id=category_id,
             bbox=bbox,
             category_name=category_name,
-            shift_amount=[0, 0],
+            shift_amount=shift_amount,
             full_shape=full_shape,
         )
 
+    @classmethod
+    def from_coco_annotation_dict(
+        cls,
+        annotation_dict: Dict,
+        full_shape: List[int],
+        category_name: str = None,
+        shift_amount: Optional[List[int]] = [0, 0],
+    ):
+        """
+        Creates ObjectAnnotation object from category name and COCO formatted
+        annotation dict (with fields "bbox", "segmentation", "category_id").
+
+        Args:
+            annotation_dict: dict
+                COCO formatted annotation dict (with fields "bbox", "segmentation", "category_id")
+            category_name: str
+                Category name of the annotation
+            full_shape: List
+                Size of the full image, should be in the form of [height, width]
+            shift_amount: List
+                To shift the box and mask predictions from sliced image to full
+                sized image, should be in the form of [shift_x, shift_y]
+        """
+        if annotation_dict["segmentation"]:
+            return cls.from_coco_segmentation(
+                segmentation=annotation_dict["segmentation"],
+                category_id=annotation_dict["category_id"],
+                category_name=category_name,
+                shift_amount=shift_amount,
+                full_shape=full_shape,
+            )
+        else:
+            return cls.from_coco_bbox(
+                bbox=annotation_dict["bbox"],
+                category_id=annotation_dict["category_id"],
+                category_name=category_name,
+                shift_amount=shift_amount,
+                full_shape=full_shape,
+            )
+
     @classmethod
     def from_shapely_annotation(
         cls,
         annotation,
-        category_id=None,
-        category_name=None,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        full_shape: List[int],
+        category_id: Optional[int] = None,
+        category_name: Optional[str] = None,
+        shift_amount: Optional[List[int]] = [0, 0],
     ):
         """
         Creates ObjectAnnotation from shapely_utils.ShapelyAnnotation
@@ -424,13 +458,13 @@ class ObjectAnnotation:
                 sized image, should be in the form of [shift_x, shift_y]
         """
         bool_mask = get_bool_mask_from_coco_segmentation(
-            annotation.to_coco_segmentation()
+            annotation.to_coco_segmentation(), width=full_shape[1], height=full_shape[0]
         )
         return cls(
             category_id=category_id,
             bool_mask=bool_mask,
             category_name=category_name,
-            shift_amount=[0, 0],
+            shift_amount=shift_amount,
             full_shape=full_shape,
         )
 
@@ -438,8 +472,8 @@ class ObjectAnnotation:
     def from_imantics_annotation(
         cls,
         annotation,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        shift_amount: Optional[List[int]] = [0, 0],
+        full_shape: Optional[List[int]] = None,
     ):
         """
         Creates ObjectAnnotation from imantics.annotation.Annotation
@@ -456,18 +490,18 @@ class ObjectAnnotation:
             category_id=annotation.category.id,
             bool_mask=annotation.mask.array,
             category_name=annotation.category.name,
-            shift_amount=[0, 0],
+            shift_amount=shift_amount,
             full_shape=full_shape,
         )
 
     def __init__(
         self,
-        bbox=None,
-        bool_mask=None,
-        category_id=None,
-        category_name=None,
-        shift_amount: list = [0, 0],
-        full_shape=None,
+        bbox: Optional[List[int]] = None,
+        bool_mask: Optional[np.ndarray] = None,
+        category_id: Optional[int] = None,
+        category_name: Optional[str] = None,
+        shift_amount: Optional[List[int]] = [0, 0],
+        full_shape: Optional[List[int]] = None,
     ):
         """
         Args:
@@ -487,9 +521,7 @@ class ObjectAnnotation:
                 the form of [height, width]
         """
         assert isinstance(category_id, int), "category_id must be an integer"
-        assert (bbox is not None) or (
-            bool_mask is not None
-        ), "you must provide a bbox or bool_mask"
+        assert (bbox is not None) or (bool_mask is not None), "you must provide a bbox or bool_mask"
 
         if bool_mask is None:
             self.mask = None
@@ -537,14 +569,14 @@ class ObjectAnnotation:
                 segmentation=self.mask.to_coco_segmentation(),
                 category_id=self.category.id,
                 category_name=self.category.name,
-                score=self.score.score,
+                score=1,
             )
         else:
             coco_prediction = CocoPrediction.from_coco_bbox(
                 bbox=self.bbox.to_coco_bbox(),
                 category_id=self.category.id,
                 category_name=self.category.name,
-                score=self.score.score,
+                score=1,
             )
         return coco_prediction
 
@@ -570,13 +602,10 @@ class ObjectAnnotation:
             import imantics
         except ImportError:
             raise ImportError(
-                'Please run "pip install -U imantics" '
-                "to install imantics first for imantics conversion."
+                'Please run "pip install -U imantics" ' "to install imantics first for imantics conversion."
             )
 
-        imantics_category = imantics.Category(
-            id=self.category.id, name=self.category.name
-        )
+        imantics_category = imantics.Category(id=self.category.id, name=self.category.name)
         if self.mask is not None:
             imantics_mask = imantics.Mask.create(self.mask.bool_mask)
             imantics_annotation = imantics.annotation.Annotation.from_mask(
